@@ -65,6 +65,8 @@ static py::list PyTokenize(const string &query) {
 		case SimplifiedTokenType::SIMPLIFIED_TOKEN_COMMENT:
 			tuple[1] = PY_SQL_TOKEN_COMMENT;
 			break;
+		default:
+			break;
 		}
 		result.append(tuple);
 	}
@@ -544,14 +546,13 @@ static void InitializeConnectionMethods(py::module_ &m) {
 	    py::arg("connection") = py::none());
 	m.def(
 	    "values",
-	    [](py::object params = py::none(), shared_ptr<DuckDBPyConnection> conn = nullptr) {
+	    [](const py::args &params, shared_ptr<DuckDBPyConnection> conn = nullptr) {
 		    if (!conn) {
 			    conn = DuckDBPyConnection::DefaultConnection();
 		    }
 		    return conn->Values(params);
 	    },
-	    "Create a relation object from the passed values", py::arg("values"), py::kw_only(),
-	    py::arg("connection") = py::none());
+	    "Create a relation object from the passed values", py::kw_only(), py::arg("connection") = py::none());
 	m.def(
 	    "table_function",
 	    [](const string &fname, py::object params = py::list(), shared_ptr<DuckDBPyConnection> conn = nullptr) {
@@ -753,46 +754,6 @@ static void InitializeConnectionMethods(py::module_ &m) {
 	    py::arg("binary_as_string") = false, py::kw_only(), py::arg("file_row_number") = false,
 	    py::arg("filename") = false, py::arg("hive_partitioning") = false, py::arg("union_by_name") = false,
 	    py::arg("compression") = py::none(), py::arg("connection") = py::none());
-	m.def(
-	    "from_substrait",
-	    [](py::bytes &proto, shared_ptr<DuckDBPyConnection> conn = nullptr) {
-		    if (!conn) {
-			    conn = DuckDBPyConnection::DefaultConnection();
-		    }
-		    return conn->FromSubstrait(proto);
-	    },
-	    "Create a query object from protobuf plan", py::arg("proto"), py::kw_only(),
-	    py::arg("connection") = py::none());
-	m.def(
-	    "get_substrait",
-	    [](const string &query, bool enable_optimizer = true, shared_ptr<DuckDBPyConnection> conn = nullptr) {
-		    if (!conn) {
-			    conn = DuckDBPyConnection::DefaultConnection();
-		    }
-		    return conn->GetSubstrait(query, enable_optimizer);
-	    },
-	    "Serialize a query to protobuf", py::arg("query"), py::kw_only(), py::arg("enable_optimizer") = true,
-	    py::arg("connection") = py::none());
-	m.def(
-	    "get_substrait_json",
-	    [](const string &query, bool enable_optimizer = true, shared_ptr<DuckDBPyConnection> conn = nullptr) {
-		    if (!conn) {
-			    conn = DuckDBPyConnection::DefaultConnection();
-		    }
-		    return conn->GetSubstraitJSON(query, enable_optimizer);
-	    },
-	    "Serialize a query to protobuf on the JSON format", py::arg("query"), py::kw_only(),
-	    py::arg("enable_optimizer") = true, py::arg("connection") = py::none());
-	m.def(
-	    "from_substrait_json",
-	    [](const string &json, shared_ptr<DuckDBPyConnection> conn = nullptr) {
-		    if (!conn) {
-			    conn = DuckDBPyConnection::DefaultConnection();
-		    }
-		    return conn->FromSubstraitJSON(json);
-	    },
-	    "Create a query object from a JSON protobuf plan", py::arg("json"), py::kw_only(),
-	    py::arg("connection") = py::none());
 	m.def(
 	    "get_table_names",
 	    [](const string &query, shared_ptr<DuckDBPyConnection> conn = nullptr) {
@@ -805,14 +766,17 @@ static void InitializeConnectionMethods(py::module_ &m) {
 	    py::arg("connection") = py::none());
 	m.def(
 	    "install_extension",
-	    [](const string &extension, bool force_install = false, shared_ptr<DuckDBPyConnection> conn = nullptr) {
+	    [](const string &extension, bool force_install = false, const py::object &repository = py::none(),
+	       const py::object &repository_url = py::none(), const py::object &version = py::none(),
+	       shared_ptr<DuckDBPyConnection> conn = nullptr) {
 		    if (!conn) {
 			    conn = DuckDBPyConnection::DefaultConnection();
 		    }
-		    conn->InstallExtension(extension, force_install);
+		    conn->InstallExtension(extension, force_install, repository, repository_url, version);
 	    },
-	    "Install an extension by name", py::arg("extension"), py::kw_only(), py::arg("force_install") = false,
-	    py::arg("connection") = py::none());
+	    "Install an extension by name, with an optional version and/or repository to get the extension from",
+	    py::arg("extension"), py::kw_only(), py::arg("force_install") = false, py::arg("repository") = py::none(),
+	    py::arg("repository_url") = py::none(), py::arg("version") = py::none(), py::arg("connection") = py::none());
 	m.def(
 	    "load_extension",
 	    [](const string &extension, shared_ptr<DuckDBPyConnection> conn = nullptr) {
@@ -1075,7 +1039,12 @@ PYBIND11_MODULE(DUCKDB_PYTHON_LIB_NAME, m) { // NOLINT
 	m.attr("__git_revision__") = DuckDB::SourceID();
 	m.attr("__interactive__") = DuckDBPyConnection::DetectAndGetEnvironment();
 	m.attr("__jupyter__") = DuckDBPyConnection::IsJupyter();
-	m.attr("default_connection") = DuckDBPyConnection::DefaultConnection();
+	m.attr("__formatted_python_version__") = DuckDBPyConnection::FormattedPythonVersion();
+	m.def("default_connection", &DuckDBPyConnection::DefaultConnection,
+	      "Retrieve the connection currently registered as the default to be used by the module");
+	m.def("set_default_connection", &DuckDBPyConnection::SetDefaultConnection,
+	      "Register the provided connection as the default to be used by the module",
+	      py::arg("connection").none(false));
 	m.attr("apilevel") = "2.0";
 	m.attr("threadsafety") = 1;
 	m.attr("paramstyle") = "qmark";
