@@ -30,7 +30,8 @@ void VerifyArrowDatasetLoaded() {
 	}
 }
 
-py::object PythonTableArrowArrayStreamFactory::ProduceScanner(py::object &arrow_scanner, py::handle &arrow_obj_handle,
+py::object PythonTableArrowArrayStreamFactory::ProduceScanner(DBConfig &config, py::object &arrow_scanner,
+                                                              py::handle &arrow_obj_handle,
                                                               ArrowStreamParameters &parameters,
                                                               const ClientProperties &client_properties) {
 	D_ASSERT(!py::isinstance<py::capsule>(arrow_obj_handle));
@@ -39,29 +40,29 @@ py::object PythonTableArrowArrayStreamFactory::ProduceScanner(py::object &arrow_
 	vector<string> unused_names;
 	vector<LogicalType> unused_types;
 	ArrowTableType arrow_table;
-	ArrowTableFunction::PopulateArrowTableType(arrow_table, schema, unused_names, unused_types);
+	ArrowTableFunction::PopulateArrowTableType(config, arrow_table, schema, unused_names, unused_types);
 
 	auto filters = parameters.filters;
 	auto &column_list = parameters.projected_columns.columns;
 	auto &filter_to_col = parameters.projected_columns.filter_to_col;
-	bool has_filter = filters && !filters->filters.empty();
 	py::list projection_list = py::cast(column_list);
+
+	bool has_filter = filters && !filters->filters.empty();
+	py::dict kwargs;
+	if (!column_list.empty()) {
+		kwargs["columns"] = projection_list;
+	}
+
 	if (has_filter) {
 		auto filter = TransformFilter(*filters, parameters.projected_columns.projection_map, filter_to_col,
 		                              client_properties, arrow_table);
-		if (column_list.empty()) {
-			return arrow_scanner(arrow_obj_handle, py::arg("filter") = filter);
-		} else {
-			return arrow_scanner(arrow_obj_handle, py::arg("columns") = projection_list, py::arg("filter") = filter);
-		}
-	} else {
-		if (column_list.empty()) {
-			return arrow_scanner(arrow_obj_handle);
-		} else {
-			return arrow_scanner(arrow_obj_handle, py::arg("columns") = projection_list);
+		if (!filter.is(py::none())) {
+			kwargs["filter"] = filter;
 		}
 	}
+	return arrow_scanner(arrow_obj_handle, **kwargs);
 }
+
 unique_ptr<ArrowArrayStreamWrapper> PythonTableArrowArrayStreamFactory::Produce(uintptr_t factory_ptr,
                                                                                 ArrowStreamParameters &parameters) {
 	py::gil_scoped_acquire acquire;
@@ -90,23 +91,26 @@ unique_ptr<ArrowArrayStreamWrapper> PythonTableArrowArrayStreamFactory::Produce(
 		auto arrow_dataset = import_cache.pyarrow.dataset().attr("dataset");
 		auto dataset = arrow_dataset(arrow_obj_handle);
 		py::object arrow_scanner = dataset.attr("__class__").attr("scanner");
-		scanner = ProduceScanner(arrow_scanner, dataset, parameters, factory->client_properties);
+		scanner = ProduceScanner(factory->config, arrow_scanner, dataset, parameters, factory->client_properties);
 		break;
 	}
 	case PyArrowObjectType::RecordBatchReader: {
-		scanner = ProduceScanner(arrow_batch_scanner, arrow_obj_handle, parameters, factory->client_properties);
+		scanner = ProduceScanner(factory->config, arrow_batch_scanner, arrow_obj_handle, parameters,
+		                         factory->client_properties);
 		break;
 	}
 	case PyArrowObjectType::Scanner: {
 		// If it's a scanner we have to turn it to a record batch reader, and then a scanner again since we can't stack
 		// scanners on arrow Otherwise pushed-down projections and filters will disappear like tears in the rain
 		auto record_batches = arrow_obj_handle.attr("to_reader")();
-		scanner = ProduceScanner(arrow_batch_scanner, record_batches, parameters, factory->client_properties);
+		scanner = ProduceScanner(factory->config, arrow_batch_scanner, record_batches, parameters,
+		                         factory->client_properties);
 		break;
 	}
 	case PyArrowObjectType::Dataset: {
 		py::object arrow_scanner = arrow_obj_handle.attr("__class__").attr("scanner");
-		scanner = ProduceScanner(arrow_scanner, arrow_obj_handle, parameters, factory->client_properties);
+		scanner =
+		    ProduceScanner(factory->config, arrow_scanner, arrow_obj_handle, parameters, factory->client_properties);
 		break;
 	}
 	default: {
@@ -274,25 +278,14 @@ py::object GetScalar(Value &constant, const string &timezone_config, const Arrow
 	case LogicalTypeId::BLOB:
 		return dataset_scalar(py::bytes(constant.GetValueUnsafe<string>()));
 	case LogicalTypeId::DECIMAL: {
-		py::object date_type = py::module_::import("pyarrow").attr("decimal128");
+		py::object decimal_type = py::module_::import("pyarrow").attr("decimal128");
 		uint8_t width;
 		uint8_t scale;
 		constant.type().GetDecimalProperties(width, scale);
-		switch (constant.type().InternalType()) {
-		case PhysicalType::INT16:
-			return dataset_scalar(scalar(constant.GetValue<int16_t>(), date_type(width, scale)));
-		case PhysicalType::INT32:
-			return dataset_scalar(scalar(constant.GetValue<int32_t>(), date_type(width, scale)));
-		case PhysicalType::INT64:
-			return dataset_scalar(scalar(constant.GetValue<int64_t>(), date_type(width, scale)));
-		default: {
-			auto hugeint_value = constant.GetValue<hugeint_t>();
-			auto hugeint_value_py = py::cast(hugeint_value.upper);
-			hugeint_value_py = hugeint_value_py.attr("__mul__")(NumericLimits<uint64_t>::Maximum());
-			hugeint_value_py = hugeint_value_py.attr("__add__")(hugeint_value.lower);
-			return dataset_scalar(scalar(hugeint_value_py, date_type(width, scale)));
-		}
-		}
+		// pyarrow only allows 'decimal.Decimal' to be used to construct decimal scalars such as 0.05
+		auto val = import_cache.decimal.Decimal()(constant.ToString());
+		return dataset_scalar(
+		    scalar(std::move(val), decimal_type(py::arg("precision") = width, py::arg("scale") = scale)));
 	}
 	default:
 		throw NotImplementedException("Unimplemented type \"%s\" for Arrow Filter Pushdown",
@@ -300,31 +293,59 @@ py::object GetScalar(Value &constant, const string &timezone_config, const Arrow
 	}
 }
 
-py::object TransformFilterRecursive(TableFilter *filter, vector<string> &column_ref, const string &timezone_config,
+py::object TransformFilterRecursive(TableFilter &filter, vector<string> column_ref, const string &timezone_config,
                                     const ArrowType &type) {
 	auto &import_cache = *DuckDBPyConnection::ImportCache();
 	py::object field = import_cache.pyarrow.dataset().attr("field");
-	switch (filter->filter_type) {
+	switch (filter.filter_type) {
 	case TableFilterType::CONSTANT_COMPARISON: {
-		auto &constant_filter = filter->Cast<ConstantFilter>();
+		auto &constant_filter = filter.Cast<ConstantFilter>();
 		auto constant_field = field(py::tuple(py::cast(column_ref)));
 		auto constant_value = GetScalar(constant_filter.constant, timezone_config, type);
+
+		bool is_nan = false;
+		auto &constant = constant_filter.constant;
+		auto &constant_type = constant.type();
+		if (constant_type.id() == LogicalTypeId::FLOAT) {
+			is_nan = Value::IsNan(constant.GetValue<float>());
+		} else if (constant_type.id() == LogicalTypeId::DOUBLE) {
+			is_nan = Value::IsNan(constant.GetValue<double>());
+		}
+
+		// Special handling for NaN comparisons (to explicitly violate IEEE-754)
+		if (is_nan) {
+			switch (constant_filter.comparison_type) {
+			case ExpressionType::COMPARE_EQUAL:
+			case ExpressionType::COMPARE_GREATERTHANOREQUALTO:
+				return constant_field.attr("is_nan")();
+			case ExpressionType::COMPARE_LESSTHAN:
+			case ExpressionType::COMPARE_NOTEQUAL:
+				return constant_field.attr("is_nan")().attr("__invert__")();
+			case ExpressionType::COMPARE_GREATERTHAN:
+				// Nothing is greater than NaN
+				return import_cache.pyarrow.dataset().attr("scalar")(false);
+			case ExpressionType::COMPARE_LESSTHANOREQUALTO:
+				// Everything is less than or equal to NaN
+				return import_cache.pyarrow.dataset().attr("scalar")(true);
+			default:
+				throw NotImplementedException("Unsupported comparison type (%s) for NaN values",
+				                              EnumUtil::ToString(constant_filter.comparison_type));
+			}
+		}
+
 		switch (constant_filter.comparison_type) {
-		case ExpressionType::COMPARE_EQUAL: {
+		case ExpressionType::COMPARE_EQUAL:
 			return constant_field.attr("__eq__")(constant_value);
-		}
-		case ExpressionType::COMPARE_LESSTHAN: {
+		case ExpressionType::COMPARE_LESSTHAN:
 			return constant_field.attr("__lt__")(constant_value);
-		}
-		case ExpressionType::COMPARE_GREATERTHAN: {
+		case ExpressionType::COMPARE_GREATERTHAN:
 			return constant_field.attr("__gt__")(constant_value);
-		}
-		case ExpressionType::COMPARE_LESSTHANOREQUALTO: {
+		case ExpressionType::COMPARE_LESSTHANOREQUALTO:
 			return constant_field.attr("__le__")(constant_value);
-		}
-		case ExpressionType::COMPARE_GREATERTHANOREQUALTO: {
+		case ExpressionType::COMPARE_GREATERTHANOREQUALTO:
 			return constant_field.attr("__ge__")(constant_value);
-		}
+		case ExpressionType::COMPARE_NOTEQUAL:
+			return constant_field.attr("__ne__")(constant_value);
 		default:
 			throw NotImplementedException("Comparison Type can't be an Arrow Scan Pushdown Filter");
 		}
@@ -340,44 +361,52 @@ py::object TransformFilterRecursive(TableFilter *filter, vector<string> &column_
 	}
 	//! We do not pushdown or conjunctions yet
 	case TableFilterType::CONJUNCTION_OR: {
-		idx_t i = 0;
-		auto &or_filter = filter->Cast<ConjunctionOrFilter>();
-		//! Get first non null filter type
-		auto child_filter = or_filter.child_filters[i++].get();
-		py::object expression = TransformFilterRecursive(child_filter, column_ref, timezone_config, type);
-		while (i < or_filter.child_filters.size()) {
-			child_filter = or_filter.child_filters[i++].get();
+		auto &or_filter = filter.Cast<ConjunctionOrFilter>();
+		py::object expression = py::none();
+		for (idx_t i = 0; i < or_filter.child_filters.size(); i++) {
+			auto &child_filter = *or_filter.child_filters[i];
 			py::object child_expression = TransformFilterRecursive(child_filter, column_ref, timezone_config, type);
-			expression = expression.attr("__or__")(child_expression);
+			if (child_expression.is(py::none())) {
+				continue;
+			}
+			if (expression.is(py::none())) {
+				expression = std::move(child_expression);
+			} else {
+				expression = expression.attr("__or__")(child_expression);
+			}
 		}
 		return expression;
 	}
 	case TableFilterType::CONJUNCTION_AND: {
-		idx_t i = 0;
-		auto &and_filter = filter->Cast<ConjunctionAndFilter>();
-		auto child_filter = and_filter.child_filters[i++].get();
-		py::object expression = TransformFilterRecursive(child_filter, column_ref, timezone_config, type);
-		while (i < and_filter.child_filters.size()) {
-			child_filter = and_filter.child_filters[i++].get();
+		auto &and_filter = filter.Cast<ConjunctionAndFilter>();
+		py::object expression = py::none();
+		for (idx_t i = 0; i < and_filter.child_filters.size(); i++) {
+			auto &child_filter = *and_filter.child_filters[i];
 			py::object child_expression = TransformFilterRecursive(child_filter, column_ref, timezone_config, type);
-			expression = expression.attr("__and__")(child_expression);
+			if (child_expression.is(py::none())) {
+				continue;
+			}
+			if (expression.is(py::none())) {
+				expression = std::move(child_expression);
+			} else {
+				expression = expression.attr("__and__")(child_expression);
+			}
 		}
 		return expression;
 	}
 	case TableFilterType::STRUCT_EXTRACT: {
-		auto &struct_filter = filter->Cast<StructFilter>();
-		auto &child_type = StructType::GetChildType(type.GetDuckType(), struct_filter.child_idx);
+		auto &struct_filter = filter.Cast<StructFilter>();
 		auto &child_name = struct_filter.child_name;
 		auto &struct_type_info = type.GetTypeInfo<ArrowStructInfo>();
 		auto &struct_child_type = struct_type_info.GetChild(struct_filter.child_idx);
 
 		column_ref.push_back(child_name);
-		auto child_expr =
-		    TransformFilterRecursive(struct_filter.child_filter.get(), column_ref, timezone_config, struct_child_type);
-		column_ref.pop_back();
-
+		auto child_expr = TransformFilterRecursive(*struct_filter.child_filter, std::move(column_ref), timezone_config,
+		                                           struct_child_type);
 		return child_expr;
 	}
+	case TableFilterType::OPTIONAL_FILTER:
+		return py::none();
 	default:
 		throw NotImplementedException("Pushdown Filter Type not supported in Arrow Scans");
 	}
@@ -388,22 +417,27 @@ py::object PythonTableArrowArrayStreamFactory::TransformFilter(TableFilterSet &f
                                                                unordered_map<idx_t, idx_t> filter_to_col,
                                                                const ClientProperties &config,
                                                                const ArrowTableType &arrow_table) {
-	auto filters_map = &filter_collection.filters;
-	auto it = filters_map->begin();
-	D_ASSERT(columns.find(it->first) != columns.end());
-	auto arrow_type = &arrow_table.GetColumns().at(filter_to_col.at(it->first));
+	auto &filters_map = filter_collection.filters;
 
-	vector<string> column_ref;
-	column_ref.push_back(columns[it->first]);
-	py::object expression = TransformFilterRecursive(it->second.get(), column_ref, config.time_zone, **arrow_type);
-	while (it != filters_map->end()) {
-		arrow_type = &arrow_table.GetColumns().at(filter_to_col.at(it->first));
-		column_ref.clear();
-		column_ref.push_back(columns[it->first]);
-		py::object child_expression =
-		    TransformFilterRecursive(it->second.get(), column_ref, config.time_zone, **arrow_type);
-		expression = expression.attr("__and__")(child_expression);
-		it++;
+	py::object expression = py::none();
+	for (auto &it : filters_map) {
+		auto column_idx = it.first;
+		auto &column_name = columns[column_idx];
+
+		vector<string> column_ref;
+		column_ref.push_back(column_name);
+
+		D_ASSERT(columns.find(column_idx) != columns.end());
+
+		auto &arrow_type = arrow_table.GetColumns().at(filter_to_col.at(column_idx));
+		py::object child_expression = TransformFilterRecursive(*it.second, column_ref, config.time_zone, *arrow_type);
+		if (child_expression.is(py::none())) {
+			continue;
+		} else if (expression.is(py::none())) {
+			expression = std::move(child_expression);
+		} else {
+			expression = expression.attr("__and__")(child_expression);
+		}
 	}
 	return expression;
 }
