@@ -7,6 +7,7 @@
 #include "duckdb/main/client_context.hpp"
 #include "duckdb_python/numpy/numpy_type.hpp"
 #include "duckdb/main/relation/query_relation.hpp"
+#include "duckdb/main/relation/join_relation.hpp"
 #include "duckdb/parser/parser.hpp"
 #include "duckdb/main/relation/view_relation.hpp"
 #include "duckdb/function/pragma/pragma_functions.hpp"
@@ -19,6 +20,7 @@
 #include "duckdb/main/relation/value_relation.hpp"
 #include "duckdb/main/relation/filter_relation.hpp"
 #include "duckdb_python/expression/pyexpression.hpp"
+#include "duckdb/common/arrow/physical_arrow_collector.hpp"
 
 namespace duckdb {
 
@@ -43,7 +45,7 @@ bool DuckDBPyRelation::CanBeRegisteredBy(ClientContext &context) {
 		// PyRelation without an internal relation can not be registered
 		return false;
 	}
-	auto this_context = rel->context.TryGetContext();
+	auto this_context = rel->context->TryGetContext();
 	if (!this_context) {
 		return false;
 	}
@@ -58,10 +60,8 @@ bool DuckDBPyRelation::CanBeRegisteredBy(shared_ptr<ClientContext> &con) {
 }
 
 DuckDBPyRelation::~DuckDBPyRelation() {
-	// FIXME: It makes sense to release the GIL here, but it causes a crash
-	// because pybind11's gil_scoped_acquire and gil_scoped_release can not be nested
-	// The Relation will need to call the destructor of the ExternalDependency, which might need to hold the GIL
-	// py::gil_scoped_release gil;
+	D_ASSERT(py::gil_check());
+	py::gil_scoped_release gil;
 	rel.reset();
 }
 
@@ -127,7 +127,7 @@ unique_ptr<DuckDBPyRelation> DuckDBPyRelation::ProjectFromTypes(const py::object
 		LogicalType type;
 		if (py::isinstance<py::str>(item)) {
 			string type_str = py::str(item);
-			type = TransformStringToLogicalType(type_str, *rel->context.GetContext());
+			type = TransformStringToLogicalType(type_str, *rel->context->GetContext());
 		} else if (py::isinstance<DuckDBPyType>(item)) {
 			auto *type_p = item.cast<DuckDBPyType *>();
 			type = type_p->Type();
@@ -254,7 +254,7 @@ vector<unique_ptr<ParsedExpression>> GetExpressions(ClientContext &context, cons
 
 unique_ptr<DuckDBPyRelation> DuckDBPyRelation::Aggregate(const py::object &expr, const string &groups) {
 	AssertRelation();
-	auto expressions = GetExpressions(*rel->context.GetContext(), expr);
+	auto expressions = GetExpressions(*rel->context->GetContext(), expr);
 	if (!groups.empty()) {
 		return make_uniq<DuckDBPyRelation>(rel->Aggregate(std::move(expressions), groups));
 	}
@@ -779,7 +779,8 @@ static unique_ptr<QueryResult> PyExecuteRelation(const shared_ptr<Relation> &rel
 	if (!rel) {
 		return nullptr;
 	}
-	auto context = rel->context.GetContext();
+	auto context = rel->context->GetContext();
+	D_ASSERT(py::gil_check());
 	py::gil_scoped_release release;
 	auto pending_query = context->PendingQuery(rel, stream_result);
 	return DuckDBPyConnection::CompletePendingQuery(*pending_query);
@@ -791,6 +792,7 @@ unique_ptr<QueryResult> DuckDBPyRelation::ExecuteInternal(bool stream_result) {
 }
 
 void DuckDBPyRelation::ExecuteOrThrow(bool stream_result) {
+	py::gil_scoped_acquire gil;
 	result.reset();
 	auto query_result = ExecuteInternal(stream_result);
 	if (!query_result) {
@@ -934,6 +936,15 @@ duckdb::pyarrow::Table DuckDBPyRelation::ToArrowTableInternal(idx_t batch_size,
 		if (!rel) {
 			return py::none();
 		}
+		auto &config = ClientConfig::GetConfig(*rel->context->GetContext());
+		ScopedConfigSetting scoped_setting(
+		    config,
+		    [&batch_size](ClientConfig &config) {
+			    config.result_collector = [&batch_size](ClientContext &context, PreparedStatementData &data) {
+				    return PhysicalArrowCollector::Create(context, data, batch_size);
+			    };
+		    },
+		    [](ClientConfig &config) { config.result_collector = nullptr; });
 		ExecuteOrThrow();
 	}
 	AssertResultOpen();
@@ -946,7 +957,7 @@ duckdb::pyarrow::Table DuckDBPyRelation::ToArrowTable(idx_t batch_size) {
 	return ToArrowTableInternal(batch_size, false);
 }
 
-py::object DuckDBPyRelation::ToArrowCapsule() {
+py::object DuckDBPyRelation::ToArrowCapsule(const py::object &requested_schema) {
 	if (!result) {
 		if (!rel) {
 			return py::none();
@@ -1012,13 +1023,27 @@ unique_ptr<DuckDBPyRelation> DuckDBPyRelation::GetAttribute(const string &name)
 		throw py::attribute_error(
 		    StringUtil::Format("This relation does not contain a column by the name of '%s'", name));
 	}
+	vector<string> column_names;
 	if (names.size() == 1 && ContainsStructFieldByName(types[0], name)) {
-		return make_uniq<DuckDBPyRelation>(rel->Project({StringUtil::Format("%s.%s", names[0], name)}));
+		// e.g 'rel['my_struct']['my_field']:
+		// first 'my_struct' is selected by the bottom condition
+		// then 'my_field' is accessed on the result of this
+		column_names.push_back(names[0]);
+		column_names.push_back(name);
+	} else if (ContainsColumnByName(name)) {
+		column_names.push_back(name);
 	}
-	if (ContainsColumnByName(name)) {
-		return make_uniq<DuckDBPyRelation>(rel->Project({StringUtil::Format("\"%s\"", name)}));
+
+	if (column_names.empty()) {
+		throw py::attribute_error(
+		    StringUtil::Format("This relation does not contain a column by the name of '%s'", name));
 	}
-	throw py::attribute_error(StringUtil::Format("This relation does not contain a column by the name of '%s'", name));
+
+	vector<unique_ptr<ParsedExpression>> expressions;
+	expressions.push_back(std::move(make_uniq<ColumnRefExpression>(column_names)));
+	vector<string> aliases;
+	aliases.push_back(name);
+	return make_uniq<DuckDBPyRelation>(rel->Project(std::move(expressions), aliases));
 }
 
 unique_ptr<DuckDBPyRelation> DuckDBPyRelation::Union(DuckDBPyRelation *other) {
@@ -1075,12 +1100,12 @@ static JoinType ParseJoinType(const string &type) {
 unique_ptr<DuckDBPyRelation> DuckDBPyRelation::Join(DuckDBPyRelation *other, const py::object &condition,
                                                     const string &type) {
 
-	JoinType dtype;
+	JoinType join_type;
 	string type_string = StringUtil::Lower(type);
 	StringUtil::Trim(type_string);
 
-	dtype = ParseJoinType(type_string);
-	if (dtype == JoinType::INVALID) {
+	join_type = ParseJoinType(type_string);
+	if (join_type == JoinType::INVALID) {
 		ThrowUnsupportedJoinTypeError(type);
 	}
 	auto alias = GetAlias();
@@ -1091,7 +1116,23 @@ unique_ptr<DuckDBPyRelation> DuckDBPyRelation::Join(DuckDBPyRelation *other, con
 	}
 	if (py::isinstance<py::str>(condition)) {
 		auto condition_string = std::string(py::cast<py::str>(condition));
-		return make_uniq<DuckDBPyRelation>(rel->Join(other->rel, condition_string, dtype));
+		return make_uniq<DuckDBPyRelation>(rel->Join(other->rel, condition_string, join_type));
+	}
+	vector<string> using_list;
+	if (py::is_list_like(condition)) {
+		auto using_list_p = py::list(condition);
+		for (auto &item : using_list_p) {
+			if (!py::isinstance<py::str>(item)) {
+				string actual_type = py::str(item.get_type());
+				throw InvalidInputException("Using clause should be a list of strings, not %s", actual_type);
+			}
+			using_list.push_back(std::string(py::str(item)));
+		}
+		if (using_list.empty()) {
+			throw InvalidInputException("Please provide at least one string in the condition to create a USING clause");
+		}
+		auto join_relation = make_shared_ptr<JoinRelation>(rel, other->rel, std::move(using_list), join_type);
+		return make_uniq<DuckDBPyRelation>(std::move(join_relation));
 	}
 	shared_ptr<DuckDBPyExpression> condition_expr;
 	if (!py::try_cast(condition, condition_expr)) {
@@ -1100,7 +1141,11 @@ unique_ptr<DuckDBPyRelation> DuckDBPyRelation::Join(DuckDBPyRelation *other, con
 	}
 	vector<unique_ptr<ParsedExpression>> conditions;
 	conditions.push_back(condition_expr->GetExpression().Copy());
-	return make_uniq<DuckDBPyRelation>(rel->Join(other->rel, std::move(conditions), dtype));
+	return make_uniq<DuckDBPyRelation>(rel->Join(other->rel, std::move(conditions), join_type));
+}
+
+unique_ptr<DuckDBPyRelation> DuckDBPyRelation::Cross(DuckDBPyRelation *other) {
+	return make_uniq<DuckDBPyRelation>(rel->CrossProduct(other->rel));
 }
 
 static Value NestedDictToStruct(const py::object &dictionary) {
@@ -1132,7 +1177,10 @@ static Value NestedDictToStruct(const py::object &dictionary) {
 }
 
 void DuckDBPyRelation::ToParquet(const string &filename, const py::object &compression, const py::object &field_ids,
-                                 const py::object &row_group_size_bytes, const py::object &row_group_size) {
+                                 const py::object &row_group_size_bytes, const py::object &row_group_size,
+                                 const py::object &overwrite, const py::object &per_thread_output,
+                                 const py::object &use_tmp_file, const py::object &partition_by,
+                                 const py::object &write_partition_columns, const py::object &append) {
 	case_insensitive_map_t<vector<Value>> options;
 
 	if (!py::none().is(compression)) {
@@ -1173,6 +1221,56 @@ void DuckDBPyRelation::ToParquet(const string &filename, const py::object &compr
 		options["row_group_size"] = {Value(row_group_size_int)};
 	}
 
+	if (!py::none().is(partition_by)) {
+		if (!py::isinstance<py::list>(partition_by)) {
+			throw InvalidInputException("to_parquet only accepts 'partition_by' as a list of strings");
+		}
+		vector<Value> partition_by_values;
+		const py::list &partition_fields = partition_by;
+		for (auto &field : partition_fields) {
+			if (!py::isinstance<py::str>(field)) {
+				throw InvalidInputException("to_parquet only accepts 'partition_by' as a list of strings");
+			}
+			partition_by_values.emplace_back(Value(py::str(field)));
+		}
+		options["partition_by"] = {partition_by_values};
+	}
+
+	if (!py::none().is(write_partition_columns)) {
+		if (!py::isinstance<py::bool_>(write_partition_columns)) {
+			throw InvalidInputException("to_parquet only accepts 'write_partition_columns' as a boolean");
+		}
+		options["write_partition_columns"] = {Value::BOOLEAN(py::bool_(write_partition_columns))};
+	}
+
+	if (!py::none().is(append)) {
+		if (!py::isinstance<py::bool_>(append)) {
+			throw InvalidInputException("to_parquet only accepts 'append' as a boolean");
+		}
+		options["append"] = {Value::BOOLEAN(py::bool_(append))};
+	}
+
+	if (!py::none().is(overwrite)) {
+		if (!py::isinstance<py::bool_>(overwrite)) {
+			throw InvalidInputException("to_parquet only accepts 'overwrite' as a boolean");
+		}
+		options["overwrite_or_ignore"] = {Value::BOOLEAN(py::bool_(overwrite))};
+	}
+
+	if (!py::none().is(per_thread_output)) {
+		if (!py::isinstance<py::bool_>(per_thread_output)) {
+			throw InvalidInputException("to_parquet only accepts 'per_thread_output' as a boolean");
+		}
+		options["per_thread_output"] = {Value::BOOLEAN(py::bool_(per_thread_output))};
+	}
+
+	if (!py::none().is(use_tmp_file)) {
+		if (!py::isinstance<py::bool_>(use_tmp_file)) {
+			throw InvalidInputException("to_parquet only accepts 'use_tmp_file' as a boolean");
+		}
+		options["use_tmp_file"] = {Value::BOOLEAN(py::bool_(use_tmp_file))};
+	}
+
 	auto write_parquet = rel->WriteParquetRel(filename, std::move(options));
 	PyExecuteRelation(write_parquet);
 }
@@ -1342,7 +1440,7 @@ unique_ptr<DuckDBPyRelation> DuckDBPyRelation::Query(const string &view_name, co
 	auto view_relation = CreateView(view_name);
 	auto all_dependencies = rel->GetAllDependencies();
 
-	Parser parser(rel->context.GetContext()->GetParserOptions());
+	Parser parser(rel->context->GetContext()->GetParserOptions());
 	parser.ParseQuery(sql_query);
 	if (parser.statements.size() != 1) {
 		throw InvalidInputException("'DuckDBPyRelation.query' only accepts a single statement");
@@ -1350,7 +1448,7 @@ unique_ptr<DuckDBPyRelation> DuckDBPyRelation::Query(const string &view_name, co
 	auto &statement = *parser.statements[0];
 	if (statement.type == StatementType::SELECT_STATEMENT) {
 		auto select_statement = unique_ptr_cast<SQLStatement, SelectStatement>(std::move(parser.statements[0]));
-		auto query_relation = make_shared_ptr<QueryRelation>(rel->context.GetContext(), std::move(select_statement),
+		auto query_relation = make_shared_ptr<QueryRelation>(rel->context->GetContext(), std::move(select_statement),
 		                                                     sql_query, "query_relation");
 		return make_uniq<DuckDBPyRelation>(std::move(query_relation));
 	} else if (IsDescribeStatement(statement)) {
@@ -1358,8 +1456,9 @@ unique_ptr<DuckDBPyRelation> DuckDBPyRelation::Query(const string &view_name, co
 		return Query(view_name, query);
 	}
 	{
+		D_ASSERT(py::gil_check());
 		py::gil_scoped_release release;
-		auto query_result = rel->context.GetContext()->Query(std::move(parser.statements[0]), false);
+		auto query_result = rel->context->GetContext()->Query(std::move(parser.statements[0]), false);
 		// Execute it anyways, for creation/altering statements
 		// We only care that it succeeds, we can't store the result
 		D_ASSERT(query_result);
@@ -1387,6 +1486,50 @@ static bool IsAcceptedInsertRelationType(const Relation &relation) {
 	return relation.type == RelationType::TABLE_RELATION;
 }
 
+void DuckDBPyRelation::Update(const py::object &set_p, const py::object &where) {
+	AssertRelation();
+	unique_ptr<ParsedExpression> condition;
+	if (!py::none().is(where)) {
+		shared_ptr<DuckDBPyExpression> py_expr;
+		if (!py::try_cast<shared_ptr<DuckDBPyExpression>>(where, py_expr)) {
+			throw InvalidInputException("Please provide an Expression to 'condition'");
+		}
+		condition = py_expr->GetExpression().Copy();
+	}
+
+	if (!py::is_dict_like(set_p)) {
+		throw InvalidInputException("Please provide 'set' as a dictionary of column name to Expression");
+	}
+
+	vector<string> names;
+	vector<unique_ptr<ParsedExpression>> expressions;
+
+	py::dict set = py::dict(set_p);
+	auto arg_count = set.size();
+	if (arg_count == 0) {
+		throw InvalidInputException("Please provide at least one set expression");
+	}
+
+	for (auto item : set) {
+		py::object item_key = item.first.cast<py::object>();
+		py::object item_value = item.second.cast<py::object>();
+
+		if (!py::isinstance<py::str>(item_key)) {
+			throw InvalidInputException("Please provide the column name as the key of the dictionary");
+		}
+		shared_ptr<DuckDBPyExpression> py_expr;
+		if (!py::try_cast<shared_ptr<DuckDBPyExpression>>(item_value, py_expr)) {
+			string actual_type = py::str(item_value.get_type());
+			throw InvalidInputException("Please provide an object of type Expression as the value, not %s",
+			                            actual_type);
+		}
+		names.push_back(std::string(py::str(item_key)));
+		expressions.push_back(py_expr->GetExpression().Copy());
+	}
+
+	return rel->Update(std::move(names), std::move(expressions), std::move(condition));
+}
+
 void DuckDBPyRelation::Insert(const py::object &params) {
 	AssertRelation();
 	if (!IsAcceptedInsertRelationType(*this->rel)) {
@@ -1394,6 +1537,7 @@ void DuckDBPyRelation::Insert(const py::object &params) {
 	}
 	vector<vector<Value>> values {DuckDBPyConnection::TransformPythonParamList(params)};
 
+	D_ASSERT(py::gil_check());
 	py::gil_scoped_release release;
 	rel->Insert(values);
 }
@@ -1425,7 +1569,7 @@ string DuckDBPyRelation::ToStringInternal(const BoxRendererConfig &config, bool
 		auto limit = Limit(config.limit, 0);
 		auto res = limit->ExecuteInternal();
 
-		auto context = rel->context.GetContext();
+		auto context = rel->context->GetContext();
 		rendered_result = res->ToBox(*context, config);
 	}
 	return rendered_result;
@@ -1481,8 +1625,8 @@ void DuckDBPyRelation::Print(const Optional<py::int_> &max_width, const Optional
 	py::print(py::str(ToStringInternal(config, invalidate_cache)));
 }
 
-static ExplainFormat GetExplainFormat() {
-	if (DuckDBPyConnection::IsJupyter()) {
+static ExplainFormat GetExplainFormat(ExplainType type) {
+	if (DuckDBPyConnection::IsJupyter() && type != ExplainType::EXPLAIN_ANALYZE) {
 		return ExplainFormat::HTML;
 	} else {
 		return ExplainFormat::DEFAULT;
@@ -1500,9 +1644,10 @@ static void DisplayHTML(const string &html) {
 
 string DuckDBPyRelation::Explain(ExplainType type) {
 	AssertRelation();
+	D_ASSERT(py::gil_check());
 	py::gil_scoped_release release;
 
-	auto explain_format = GetExplainFormat();
+	auto explain_format = GetExplainFormat(type);
 	auto res = rel->Explain(type, explain_format);
 	D_ASSERT(res->type == duckdb::QueryResultType::MATERIALIZED_RESULT);
 	auto &materialized = res->Cast<MaterializedQueryResult>();
