@@ -16,23 +16,25 @@
 #include "duckdb_python/arrow/arrow_export_utils.hpp"
 #include "duckdb/common/types/arrow_aux_data.hpp"
 #include "duckdb/parser/tableref/table_function_ref.hpp"
+#include "duckdb/function/table/arrow/arrow_duck_schema.hpp"
 
 namespace duckdb {
 
 static py::list ConvertToSingleBatch(vector<LogicalType> &types, vector<string> &names, DataChunk &input,
-                                     const ClientProperties &options) {
+                                     ClientProperties &options, ClientContext &context) {
 	ArrowSchema schema;
 	ArrowConverter::ToArrowSchema(&schema, types, names, options);
 
 	py::list single_batch;
-	ArrowAppender appender(types, STANDARD_VECTOR_SIZE, options);
+	ArrowAppender appender(types, STANDARD_VECTOR_SIZE, options,
+	                       ArrowTypeExtensionData::GetExtensionTypes(context, types));
 	appender.Append(input, 0, input.size(), input.size());
 	auto array = appender.Finalize();
 	TransformDuckToArrowChunk(schema, array, single_batch);
 	return single_batch;
 }
 
-static py::object ConvertDataChunkToPyArrowTable(DataChunk &input, const ClientProperties &options) {
+static py::object ConvertDataChunkToPyArrowTable(DataChunk &input, ClientProperties &options, ClientContext &context) {
 	auto types = input.GetTypes();
 	vector<string> names;
 	names.reserve(types.size());
@@ -40,7 +42,7 @@ static py::object ConvertDataChunkToPyArrowTable(DataChunk &input, const ClientP
 		names.push_back(StringUtil::Format("c%d", i));
 	}
 
-	return pyarrow::ToArrowTable(types, names, ConvertToSingleBatch(types, names, input, options), options);
+	return pyarrow::ToArrowTable(types, names, ConvertToSingleBatch(types, names, input, options, context), options);
 }
 
 // If these types are arrow canonical extensions, we must check if they are registered.
@@ -68,9 +70,11 @@ void AreExtensionsRegistered(const LogicalType &arrow_type, const LogicalType &d
 static void ConvertArrowTableToVector(const py::object &table, Vector &out, ClientContext &context, idx_t count) {
 	// Create the stream factory from the Table object
 	auto ptr = table.ptr();
+	D_ASSERT(py::gil_check());
 	py::gil_scoped_release gil;
 
-	auto stream_factory = make_uniq<PythonTableArrowArrayStreamFactory>(ptr, context.GetClientProperties());
+	auto stream_factory =
+	    make_uniq<PythonTableArrowArrayStreamFactory>(ptr, context.GetClientProperties(), DBConfig::GetConfig(context));
 	auto stream_factory_produce = PythonTableArrowArrayStreamFactory::Produce;
 	auto stream_factory_get_schema = PythonTableArrowArrayStreamFactory::GetSchema;
 
@@ -125,6 +129,7 @@ static void ConvertArrowTableToVector(const py::object &table, Vector &out, Clie
 
 	VectorOperations::Cast(context, result.data[0], out, count);
 	out.Flatten(count);
+	out.Verify(count);
 }
 
 static string NullHandlingError() {
@@ -172,12 +177,12 @@ static scalar_function_t CreateVectorizedFunction(PyObject *function, PythonExce
 		// owning references
 		py::object python_object;
 		// Convert the input datachunk to pyarrow
-		ClientProperties options;
+		//		ClientProperties options;
 
-		if (state.HasContext()) {
-			auto &context = state.GetContext();
-			options = context.GetClientProperties();
-		}
+		//		if (state.HasContext()) {
+		auto &context = state.GetContext();
+		auto options = context.GetClientProperties();
+		//		}
 
 		auto result_validity = FlatVector::Validity(result);
 		SelectionVector selvec(input.size());
@@ -209,7 +214,7 @@ static scalar_function_t CreateVectorizedFunction(PyObject *function, PythonExce
 			}
 		}
 
-		auto pyarrow_table = ConvertDataChunkToPyArrowTable(input, options);
+		auto pyarrow_table = ConvertDataChunkToPyArrowTable(input, options, state.GetContext());
 		py::tuple column_list = pyarrow_table.attr("columns");
 
 		auto count = input.size();
@@ -373,6 +378,17 @@ struct ParameterKind {
 	}
 };
 
+static bool NumpyDeprecatesAccessToCore(const py::tuple &numpy_version) {
+	if (numpy_version.empty()) {
+		return false;
+	}
+	if (string(py::str(numpy_version[0])) == string("2")) {
+		//! Starting with numpy version 2.0.0 the use of 'core' is deprecated.
+		return true;
+	}
+	return false;
+}
+
 struct PythonUDFData {
 public:
 	PythonUDFData(const string &name, bool vectorized, FunctionNullHandling null_handling)
@@ -475,9 +491,21 @@ public:
 	ScalarFunction GetFunction(const py::function &udf, PythonExceptionHandling exception_handling, bool side_effects,
 	                           const ClientProperties &client_properties) {
 
-		auto &import_cache = *DuckDBPyConnection::ImportCache();
 		// Import this module, because importing this from a non-main thread causes a segfault
-		(void)import_cache.numpy.core.multiarray();
+
+		auto &import_cache = *DuckDBPyConnection::ImportCache();
+		py::handle core;
+		auto numpy = import_cache.numpy();
+		if (!numpy) {
+			throw InvalidInputException("'numpy' is required for this operation, but it wasn't installed");
+		}
+		auto numpy_version = py::cast<py::tuple>(numpy.attr("__version__"));
+		if (NumpyDeprecatesAccessToCore(numpy_version)) {
+			core = numpy.attr("_core");
+		} else {
+			core = numpy.attr("core");
+		}
+		(void)core.attr("multiarray");
 
 		scalar_function_t func;
 		if (vectorized) {
