@@ -26,3 +26,9 @@ class TestNumpyDatetime64(object):
         res1 = duckdb_con.execute("select * from test").fetchnumpy()
         date_value = {'date': np.array(['2263-02-28'], dtype='datetime64[us]')}
         assert res1 == date_value
+
+    def test_numpy_enum_conversion(self, duckdb_cursor):
+        arr = np.array(['a', 'b', 'c'])
+        rel = duckdb_cursor.sql("select * from arr")
+        res = rel.fetchnumpy()['column0']
+        np.testing.assert_equal(res, arr)
