@@ -807,7 +807,7 @@ void Linenoise::RefreshMultiLine() {
 	int rows, cols;
 	int new_cursor_row, new_cursor_x;
 	PositionToColAndRow(pos, new_cursor_row, new_cursor_x, rows, cols);
-	int col; /* colum position, zero-based. */
+	int col; /* column position, zero-based. */
 	int old_rows = maxrows ? maxrows : 1;
 	int fd = ofd;
 	std::string highlight_buffer;
@@ -940,7 +940,7 @@ void Linenoise::RefreshMultiLine() {
 	Linenoise::Log("pos %d", pos);
 	Linenoise::Log("max cols %d", ws.ws_col);
 
-	/* Go up till we reach the expected positon. */
+	/* Go up till we reach the expected position. */
 	if (rows - new_cursor_row > 0) {
 		Linenoise::Log("go-up %d", rows - new_cursor_row);
 		snprintf(seq, 64, "\x1b[%dA", rows - new_cursor_row);
