@@ -53,6 +53,7 @@ def test_pragma(shell):
         .statement(".mode csv")
         .statement(".headers off")
         .statement(".sep |")
+        .statement('.nullvalue ""')
         .statement("CREATE TABLE t0(c0 INT);")
         .statement("PRAGMA table_info('t0');")
     )
@@ -92,24 +93,10 @@ def test_invalid_cast(shell):
     result = test.run()
     result.check_stderr("Could not convert")
 
-
-@pytest.mark.parametrize(
-    ["input", "error"],
-    [
-        (".auth ON", "sqlite3_set_authorizer"),
-        (".auth OFF", "sqlite3_set_authorizer"),
-    ],
-)
-def test_invalid_shell_commands(shell, input, error):
-    test = ShellTest(shell).statement(input)
-    result = test.run()
-    result.check_stderr(error)
-
-
 def test_invalid_backup(shell, random_filepath):
     test = ShellTest(shell).statement(f'.backup {random_filepath.as_posix()}')
     result = test.run()
-    result.check_stderr("sqlite3_backup_init")
+    result.check_stderr("unsupported in the current version of the CLI")
 
 def test_newline_in_value(shell):
     test = (
@@ -300,15 +287,6 @@ def test_execute_display(shell):
     result = test.run()
     result.check_stdout("42")
 
-# this should be fixed
-def test_selftest(shell):
-    test = (
-        ShellTest(shell)
-        .statement(".selftest")
-    )
-    result = test.run()
-    result.check_stderr("sqlite3_table_column_metadata")
-
 @pytest.mark.parametrize('generated_file', ["select 42"], indirect=True)
 def test_read(shell, generated_file):
     test = (
@@ -318,6 +296,29 @@ def test_read(shell, generated_file):
     result = test.run()
     result.check_stdout("42")
 
+@pytest.mark.parametrize('generated_file', ["select 42"], indirect=True)
+def test_execute_file(shell, generated_file):
+    test = (
+        ShellTest(shell, ['-f', generated_file.as_posix()])
+    )
+    result = test.run()
+    result.check_stdout("42")
+
+def test_execute_non_existent_file(shell):
+    test = (
+        ShellTest(shell, ['-f', '____this_file_does_not_exist'])
+    )
+    result = test.run()
+    result.check_stderr("____this_file_does_not_exist")
+
+@pytest.mark.parametrize('generated_file', ["insert into tbl values (42)"], indirect=True)
+def test_execute_files(shell, generated_file):
+    test = (
+        ShellTest(shell, ['-c', 'CREATE TABLE tbl(i INT)', '-f', generated_file.as_posix(), '-f', generated_file.as_posix(), '-c', 'SELECT SUM(i) FROM tbl'])
+    )
+    result = test.run()
+    result.check_stdout("84")
+
 def test_show_basic(shell):
     test = (
         ShellTest(shell)
@@ -332,7 +333,7 @@ def test_timeout(shell):
         .statement(".timeout")
     )
     result = test.run()
-    result.check_stderr("sqlite3_busy_timeout")
+    result.check_stderr("unsupported in the current version of the CLI")
 
 
 def test_save(shell, random_filepath):
@@ -341,7 +342,7 @@ def test_save(shell, random_filepath):
         .statement(f".save {random_filepath.as_posix()}")
     )
     result = test.run()
-    result.check_stderr("sqlite3_backup_init")
+    result.check_stderr("unsupported in the current version of the CLI")
 
 def test_restore(shell, random_filepath):
     test = (
@@ -349,7 +350,7 @@ def test_restore(shell, random_filepath):
         .statement(f".restore {random_filepath.as_posix()}")
     )
     result = test.run()
-    result.check_stderr("sqlite3_backup_init")
+    result.check_stderr("unsupported in the current version of the CLI")
 
 @pytest.mark.parametrize("cmd", [
     ".vfsinfo",
@@ -365,26 +366,6 @@ def test_volatile_commands(shell, cmd):
     result = test.run()
     result.check_stderr("")
 
-def test_stats_error(shell):
-    test = (
-        ShellTest(shell)
-        .statement(".stats")
-    )
-    result = test.run()
-    result.check_stderr("sqlite3_status64")
-
-@pytest.mark.parametrize("param", [
-    "off",
-    "on"
-])
-def test_stats(shell, param):
-    test = (
-        ShellTest(shell)
-        .statement(f".stats {param}")
-    )
-    result = test.run()
-    result.check_stdout("")
-
 @pytest.mark.parametrize("pattern", [
     "test",
     "tes%",
@@ -401,6 +382,15 @@ def test_schema(shell, pattern):
     result = test.run()
     result.check_stdout("CREATE TABLE test(a INTEGER, b VARCHAR);")
 
+def test_schema_indent(shell):
+    test = (
+        ShellTest(shell)
+        .statement("create table test (a int, b varchar, c int, d int, k int, primary key(a, b));")
+        .statement(f".schema -indent")
+    )
+    result = test.run()
+    result.check_stdout("CREATE TABLE test(")
+
 def test_tables(shell):
     test = (
         ShellTest(shell)
@@ -519,25 +509,6 @@ def test_timer(shell):
     result = test.run()
     result.check_stdout('Run Time (s):')
 
-
-def test_scanstats(shell):
-    test = (
-        ShellTest(shell)
-        .statement(".scanstats on")
-        .statement("SELECT NULL;")
-    )
-    result = test.run()
-    result.check_stderr('scanstats')
-
-def test_trace(shell, random_filepath):
-    test = (
-        ShellTest(shell)
-        .statement(f".trace {random_filepath.as_posix()}")
-        .statement("SELECT 42;")
-    )
-    result = test.run()
-    result.check_stderr('sqlite3_trace_v2')
-    
 def test_output_csv_mode(shell, random_filepath):
     test = (
         ShellTest(shell)
@@ -614,7 +585,42 @@ def test_mode_html(shell):
     result = test.run()
     result.check_stdout('<td>fourty-two</td>')
 
-# Original comment: FIXME sqlite3_column_blob
+def test_mode_html_escapes(shell):
+    test = (
+        ShellTest(shell)
+        .statement(".mode html")
+        .statement("SELECT '<&>\"\'\'' AS \"&><\"\"\'\";")
+    )
+    result = test.run()
+    result.check_stdout('<tr><th>&amp;&gt;&lt;&quot;&#39;</th>')
+
+def test_mode_tcl_escapes(shell):
+    test = (
+        ShellTest(shell)
+        .statement(".mode tcl")
+        .statement("SELECT '<&>\"\'\'' AS \"&><\"\"\'\";")
+    )
+    result = test.run()
+    result.check_stdout('"&><\\"\'"')
+
+def test_mode_csv_escapes(shell):
+    test = (
+        ShellTest(shell)
+        .statement(".mode csv")
+        .statement("SELECT 'BEGINVAL,\n\"ENDVAL' AS \"BEGINHEADER\"\",\nENDHEADER\";")
+    )
+    result = test.run()
+    result.check_stdout('"BEGINHEADER"",\nENDHEADER"\r\n"BEGINVAL,\n""ENDVAL"')
+
+def test_mode_json_infinity(shell):
+    test = (
+        ShellTest(shell)
+        .statement(".mode json")
+        .statement("SELECT 'inf'::DOUBLE AS inf, '-inf'::DOUBLE AS ninf, 'nan'::DOUBLE AS nan, '-nan'::DOUBLE AS nnan;")
+    )
+    result = test.run()
+    result.check_stdout('[{"inf":1e999,"ninf":-1e999,"nan":null,"nnan":null}]')
+
 def test_mode_insert(shell):
     test = (
         ShellTest(shell)
@@ -828,6 +834,17 @@ def test_dump_mixed(shell):
     result = test.run()
     result.check_stdout('CREATE TABLE a(d DATE, k FLOAT, t TIMESTAMP);')
 
+def test_dump_blobs(shell):
+    test = (
+        ShellTest(shell)
+        .statement("create table test(t VARCHAR, b BLOB);")
+        .statement(".changes off")
+        .statement("insert into test values('literal blob', '\\x07\\x08\\x09');")
+        .statement(".dump")
+    )
+    result = test.run()
+    result.check_stdout("'\\x07\\x08\\x09'")
+
 def test_invalid_csv(shell, tmp_path):
     file = tmp_path / 'nonsencsv.csv'
     with open(file, 'wb+') as f:
@@ -863,18 +880,6 @@ def test_mode_trash(shell):
     result = test.run()
     result.check_stdout('')
 
-@pytest.mark.skip(reason="Broken test, ported directly, was commented out")
-def test_dump_blobs(shell):
-    test = (
-        ShellTest(shell)
-        .statement("CREATE TABLE a (b BLOB);")
-        .statement(".changes off")
-        .statement("INSERT INTO a VALUES (DATE '1992-01-01', 0.3, NOW());")
-        .statement(".dump")
-    )
-    result = test.run()
-    result.check_stdout('COMMIT')
-
 def test_sqlite_comments(shell):
     # Using /* <comment> */
     test = (
@@ -915,42 +920,6 @@ select 42;
     result = test.run()
     result.check_stdout('42')
 
-def test_sqlite_udfs_error(shell):
-    test = (
-        ShellTest(shell)
-        .statement("SELECT writefile()")
-    )
-    result = test.run()
-    result.check_stderr('wrong number of arguments to function writefile')
-
-    test = (
-        ShellTest(shell)
-        .statement("SELECT writefile('hello')")
-    )
-    result = test.run()
-    result.check_stderr('wrong number of arguments to function writefile')
-
-def test_sqlite_udfs_correct(shell, random_filepath):
-    import os
-    test = (
-        ShellTest(shell)
-        .statement(f"SELECT writefile('{random_filepath.as_posix()}', 'hello');")
-    )
-    result = test.run()
-    if not os.path.exists(random_filepath):
-        raise Exception(f"Failed to write file {random_filepath.as_posix()}")
-    with open(random_filepath, 'r') as f:
-        result.stdout = f.read()
-    result.check_stdout('hello')
-
-def test_lsmode(shell):
-    test = (
-        ShellTest(shell)
-        .statement("SELECT lsmode(1) AS lsmode;")
-    )
-    result = test.run()
-    result.check_stdout('lsmode')
-
 def test_duckbox(shell):
     test = (
         ShellTest(shell)
@@ -1036,31 +1005,46 @@ def test_nullbyte_error_rendering(shell):
     result = test.run()
     result.check_stderr('INT32')
 
-@pytest.mark.parametrize("stmt", [
-	"select sha3(NULL);"
-])
-def test_sqlite_udf_null(shell, stmt):
+def test_thousand_sep(shell):
     test = (
         ShellTest(shell)
-        .statement(stmt)
+        .statement(".thousand_sep space")
+        .statement("SELECT 10000")
+        .statement(".thousand_sep ,")
+        .statement("SELECT 10000")
+        .statement(".thousand_sep none")
+        .statement("SELECT 10000")
+        .statement(".thousand_sep")
     )
-    result = test.run()
-    result.check_stdout('NULL')
 
-def test_sqlite_udf_sha3_int(shell):
-    test = (
-        ShellTest(shell)
-        .statement("select sha3(256)")
-    )
     result = test.run()
-    result.check_stdout('A7')
+    result.check_stdout("10 000")
+    result.check_stdout("10,000")
+    result.check_stdout("10000")
+    result.check_stdout("current thousand separator")
 
-def test_sqlite_udf_sha3_non_inlined_string(shell):
+def test_decimal_sep(shell):
     test = (
         ShellTest(shell)
-        .statement("select sha3('hello world this is a long string');")
+        .statement(".decimal_sep space")
+        .statement("SELECT 10.5")
+        .statement(".decimal_sep ,")
+        .statement("SELECT 10.5")
+        .statement(".decimal_sep none")
+        .statement("SELECT 10.5")
+        .statement(".decimal_sep")
     )
+
     result = test.run()
-    result.check_stdout('D4')
+    result.check_stdout("10 5")
+    result.check_stdout("10,5")
+    result.check_stdout("10.5")
+    result.check_stdout("current decimal separator")
+
+def test_prepared_statement(shell):
+    test = ShellTest(shell).statement("select ?")
+    result = test.run()
+    result.check_stderr("Prepared statement parameters cannot be used directly")
+
 
 # fmt: on
